"""Pack to install the MariaDB database."""
import os
from typing import Literal

from jinja2 import Environment
from pydantic import BaseModel

from ou_container_builder.utils import merge_settings, render_template


class Options(BaseModel):
    pass


class Pack(BaseModel):
    name: Literal["mariadb"]
    options: Options = Options()


def process_settings(settings: dict, pack_settings: dict) -> dict:
    """Process the user-provided settings.

    :param settings: The settings parsed from the configuration file
    :type settings: dict
    :param pack_settings: The pack-specific settings parsed from the configuration file
    :type settings: dict
    :return: The updated settings
    :rtype: dict
    """
    settings = merge_settings(
        settings,
        {
            "packages": {"apt": ["mariadb-server", "sudo"]},
            "scripts": {
                "build": [
                    {
                        "commands": [
                            "mkdir -p /run/mysqld",
                            f'sed -e "s#datadir.*=.*#datadir = $HOME/mariadb#" -e "s#user.*=.*#user = {settings["image"]["user"]}#" -i /etc/mysql/mariadb.conf.d/50-server.cnf',  # noqa: E501
                            f'chown -R {settings["image"]["user"]}: /var/log/mysql /run/mysqld',
                            "chmod a+x /usr/bin/mariadb-setup.sh",
                            f'printf "{settings["image"]["user"]} ALL=NOPASSWD: /usr/bin/mariadb-setup.sh\\n" > /etc/sudoers.d/99-mariadb',  # noqa: E501
                        ]
                    },
                ],
                "startup": [{"commands": ["sudo /usr/bin/mariadb-setup.sh"]}],
            },
            "services": ["mariadb"],
            "content": [
                {"source": "/var/lib/mysql", "target": "mariadb", "overwrite": "never"},
                {
                    "source": "ou-builder-build/mariadb/mariadb-setup.sh",
                    "target": "/usr/bin/mariadb-setup.sh",
                    "overwrite": "always",
                },
            ],
        },
    )
    return settings


def generate_files(context: str, env: Environment, settings: dict, pack_settings: dict) -> None:
    """Generate the build files for the mariadb pack.

    This ensures that the the mariadb service is activated and that the configured database is set up in the user's
    home-directory.

    :param context: The context path within which the generation is running
    :type context: str
    :param env: The Jinja2 environment to use for loading and rendering templates
    :type env: :class:`~jinja2.environment.Environment`
    :param settings: The validated settings
    :type settings: dict
    :param pack_settings: The validated pack-specific settings
    :type settings: dict
    """
    render_template(
        context,
        env,
        "packs/mariadb/setup.sh",
        os.path.join("ou-builder-build", "mariadb", "mariadb-setup.sh"),
        pack_settings,
    )



"""Pack to install the Jupyterlab interface."""
from typing import Literal
from warnings import warn

from jinja2 import Environment
from pydantic import BaseModel

from ou_container_builder.utils import merge_settings


class Options(BaseModel):
    version: Literal[3] | Literal[4] = 3


class Pack(BaseModel):
    name: Literal["jupyterlab"]
    options: Options = Options()


def process_settings(settings: dict, pack_settings: dict) -> dict:
    """
    Process the user-provided settings.

    :param settings: The settings parsed from the configuration file
    :type settings: dict
    :param pack_settings: The pack-specific settings parsed from the configuration file
    :type settings: dict
    :return: The updated settings
    :rtype: dict
    """
    warn(
        "Automatically setting JupyterLab as the default is deprecated in 2.2.0 and will be removed in 3.0.0. Instead set server.default_path explicitly.",
        FutureWarning,
    )
    if pack_settings["version"] == 3:
        settings = merge_settings(
            settings,
            {
                "packages": {"pip": ["jupyterlab>=3.4.3,<4.0.0"]},
            },
        )
    elif pack_settings["version"] == 4:
        settings = merge_settings(
            settings,
            {
                "packages": {"pip": ["jupyterlab>=4.0.2,<5.0.0"]},
            },
        )
    settings = merge_settings(
        settings,
        {
            "server": {"default_path": "/lab"},
        },
    )
    return settings


def generate_files(context: str, env: Environment, settings: dict, pack_settings: dict) -> None:
    """Generate the build files for the jupyterlab pack.

    This ensures that the the jupyterlab package is installed

    :param context: The context path within which the generation is running
    :type context: str
    :param env: The Jinja2 environment to use for loading and rendering templates
    :type env: :class:`~jinja2.environment.Environment`
    :param settings: The validated settings
    :type settings: dict
    :param pack_settings: The validated pack-specific settings
    :type settings: dict
    """
    pass



"""Pack to handle scripts."""
import os
from hashlib import sha256

from jinja2 import Environment

from ou_container_builder.utils import merge_settings


def process_settings(settings: dict) -> dict:
    """Process the env core settings.

    :param settings: The settings parsed from the configuration file
    :type settings: dict
    :return: The updated settings
    :rtype: dict
    """
    if "startup" in settings["scripts"]:
        settings = merge_settings(settings, {"flags": {"ou_container_content": True}})
        new_settings = {}
        for script in settings["scripts"]["startup"]:
            if len(script["commands"]) > 1:
                hash = sha256("\n".join(script["commands"]).encode())
                script_name = f"startup-script-{hash.hexdigest()}"
                new_settings = merge_settings(
                    new_settings,
                    {
                        "content": [
                            {
                                "source": os.path.join("ou-builder-build", script_name),
                                "target": f"/usr/bin/{script_name}",
                                "overwrite": "always",
                            }
                        ],
                        "scripts": {"build": [{"commands": [f"chmod a+x /usr/bin/{script_name}"]}]},
                    },
                )
                script["commands"] = [f"/usr/bin/{script_name}"]
        settings = merge_settings(settings, new_settings)
    if "shutdown" in settings["scripts"]:
        settings = merge_settings(settings, {"flags": {"ou_container_content": True}})
        new_settings = {}
        for script in settings["scripts"]["shutdown"]:
            if len(script["commands"]) > 1:
                hash = sha256("\n".join(script["commands"]).encode())
                script_name = f"shutdown-script-{hash.hexdigest()}"
                new_settings = merge_settings(
                    new_settings,
                    {
                        "content": [
                            {
                                "source": os.path.join("ou-builder-build", script_name),
                                "target": f"/usr/bin/{script_name}",
                                "overwrite": "always",
                            }
                        ],
                        "scripts": {"build": [{"commands": [f"chmod a+x /usr/bin/{script_name}"]}]},
                    },
                )
                script["commands"] = [f"/usr/bin/{script_name}"]
        settings = merge_settings(settings, new_settings)

    return settings


def generate_files(context: str, env: Environment, settings: dict) -> dict:
    """Generate the build files for the scripts core.

    :param context: The context path within which the generation is running
    :type context: str
    :param env: The Jinja2 environment to use for loading and rendering templates
    :type env: :class:`~jinja2.environment.Environment`
    :param settings: The validated settings
    :type settings: dict
    """
    if "startup" in settings["scripts"]:
        for script in settings["scripts"]["startup"]:
            if len(script["commands"]) > 1:
                hash = sha256("\n".join(script["commands"]).encode())
                script_name = f"startup-script-{hash.hexdigest()}"
                with open(os.path.join(context, "ou-builder-build", script_name), "w") as out_f:
                    out_f.write("\n".join(script["commands"]))
    if "shutdown" in settings["scripts"]:
        for script in settings["scripts"]["shutdown"]:
            if len(script["commands"]) > 1:
                hash = sha256("\n".join(script["commands"]).encode())
                script_name = f"shutdown-script-{hash.hexdigest()}"
                with open(os.path.join(context, "ou-builder-build", script_name), "w") as out_f:
                    out_f.write("\n".join(script["commands"]))



"""Pack to handle setting up the startup process in the container.

Generates the startup script and the sudoers file to ensure the user can chown their home-directory.
"""
import os.path

from jinja2 import Environment

from ou_container_builder.utils import merge_settings, render_template


def process_settings(settings: dict) -> dict:
    """Process the startup core settings.

    :param settings: The settings parsed from the configuration file
    :type settings: dict
    :return: The updated settings
    :rtype: dict
    """
    settings = merge_settings(
        settings,
        {
            "content": [
                {"source": "ou-builder-build/startup/start.sh", "target": "/usr/bin/start.sh", "overwrite": "always"},
                {
                    "source": "ou-builder-build/startup/home-dir.sudoers",
                    "target": "/etc/sudoers.d/99-home-dir",
                    "overwrite": "always",
                },
            ],
            "scripts": {
                "build": [{"commands": ["chmod a+x /usr/bin/start.sh", "chmod 0660 /etc/sudoers.d/99-home-dir"]}]
            },
        },
    )
    return settings


def generate_files(context: str, env: Environment, settings: dict) -> dict:
    """Generate the build files for the startup core.

    :param context: The context path within which the generation is running
    :type context: str
    :param env: The Jinja2 environment to use for loading and rendering templates
    :type env: :class:`~jinja2.environment.Environment`
    :param settings: The validated settings
    :type settings: dict
    """
    # Generate the start script
    render_template(
        context, env, "core/startup/start.sh", os.path.join("ou-builder-build", "startup", "start.sh"), settings
    )
    # Generate the home-dir chown sudoers
    render_template(
        context,
        env,
        "core/startup/home-dir.sudoers",
        os.path.join("ou-builder-build", "startup", "home-dir.sudoers"),
        settings,
    )



"""Pack to install the Code Server interface."""
import os
from importlib import resources
from typing import Literal
from warnings import warn

from jinja2 import Environment
from pydantic import BaseModel

from ou_container_builder.utils import merge_settings


class Options(BaseModel):
    version: str = "4.14.1"
    extensions: list[str] = []


class Pack(BaseModel):
    name: Literal["code_server"]
    options: Options = Options()


def process_settings(settings: dict, pack_settings: dict) -> dict:
    """
    Process the user-provided settings.

    :param settings: The settings parsed from the configuration file
    :type settings: dict
    :param pack_settings: The pack-specific settings parsed from the configuration file
    :type settings: dict
    :return: The updated settings
    :rtype: dict
    """
    version = pack_settings["version"]
    warn(
        "Automatically setting the code_server as the default web-app is deprecated in 2.2.0 and will be removed in 3.0.0. Instead set server.default_path explicitly.",
        FutureWarning,
    )
    settings = merge_settings(
        settings,
        {
            "sources": {
                "apt": [
                    {
                        "name": "nodesource",
                        "key_url": "https://deb.nodesource.com/gpgkey/nodesource.gpg.key",
                        "dearmor": True,
                        "deb": {
                            "url": "https://deb.nodesource.com/node_18.x",
                            "distribution": "bullseye",
                            "component": "main",
                        },
                    }
                ],
            },
            "packages": {
                "apt": ["nodejs", "yarn"],
            },
            "scripts": {
                "build": [
                    {
                        "commands": [
                            "cd /opt",
                            f"curl -L --output code-server.tar.gz https://github.com/coder/code-server/releases/download/v{version}/code-server-{version}-linux-amd64.tar.gz",
                            "tar -zxvf code-server.tar.gz",
                            f"ln -s /opt/code-server-{version}-linux-amd64/bin/code-server /usr/local/bin",
                        ]
                    }
                ]
            },
            "content": [
                {
                    "source": "ou-builder-build/packs/code_server/logo.svg",
                    "target": "/var/lib/code_server/logo.svg",
                    "overwrite": "always",
                },
            ],
            "web_apps": [
                {
                    "path": "code-server/",
                    "command": [
                        "code-server",
                        "--auth",
                        "none",
                        "--disable-update-check",
                        "--bind-addr",
                        "0.0.0.0",
                        "--port",
                        "{port}",
                    ],
                    "timeout": 60,
                    "default": True,
                    "new_browser_tab": False,
                    "launcher_entry": {"title": "VS Code", "icon_path": "/var/lib/code_server/logo.svg"},
                }
            ],
        },
    )
    if len(pack_settings["extensions"]) > 0:
        settings = merge_settings(
            settings,
            {
                "scripts": {
                    "build": [
                        {
                            "commands": [
                                f"code-server --install-extension {extension}"
                                for extension in pack_settings["extensions"]
                            ]
                        }
                    ]
                }
            },
        )
    return settings


def generate_files(context: str, env: Environment, settings: dict, pack_settings: dict) -> None:
    """Generate the build files for the code-server pack.

    This ensures that the the code-server is installed

    :param context: The context path within which the generation is running
    :type context: str
    :param env: The Jinja2 environment to use for loading and rendering templates
    :type env: :class:`~jinja2.environment.Environment`
    :param settings: The validated settings
    :type settings: dict
    :param pack_settings: The validated pack-specific settings
    :type settings: dict
    """
    source = resources.files("ou_container_builder") / "templates" / "packs" / "code_server" / "vscode.svg"
    target_path = os.path.join(context, "ou-builder-build", "packs", "code_server", "logo.svg")
    if not os.path.exists(os.path.dirname(target_path)):
        os.makedirs(os.path.dirname(target_path), exist_ok=True)
    with resources.as_file(source) as in_path:
        with open(in_path, "rb") as in_f:
            with open(target_path, "wb") as out_f:
                out_f.write(in_f.read())



"""Pack to handle setting up the base settings.

This provides a few minimal initial settings.
"""
from jinja2 import Environment

from ou_container_builder.utils import merge_settings


def process_settings(settings: dict) -> dict:
    """Process the content core settings.

    :param settings: The settings parsed from the configuration file
    :type settings: dict
    :return: The updated settings
    :rtype: dict
    """
    settings = merge_settings(
        {
            "packages": {
                "apt": ["libcurl3-gnutls", "libcurl3-gnutls-dev", "gnutls-dev", "gcc", "build-essential"],
                "pip": ["pycurl"],
            }
        },
        settings,
    )
    return settings


def generate_files(context: str, env: Environment, settings: dict) -> None:
    """Generate the build files for the base core.

    :param context: The context path within which the generation is running
    :type context: str
    :param env: The Jinja2 environment to use for loading and rendering templates
    :type env: :class:`~jinja2.environment.Environment`
    :param settings: The validated settings
    :type settings: dict
    """
    pass



"""Configuration settings."""
import shlex
from typing import Literal, Optional

from pydantic import BaseModel, HttpUrl, constr, validator

from .packs.code_server import Pack as CodeServerPack
from .packs.jupyterlab import Pack as JupyterLabPack
from .packs.mariadb import Pack as MariaDBPack
from .packs.nbclassic import Pack as NBClassicPack
from .packs.tutorial_server import Pack as TutorialServerPack


class Module(BaseModel):
    """Settings for the module configuration."""

    code: constr(min_length=1)
    presentation: constr(min_length=1)


class Image(BaseModel):
    """Settings for the Docker image configuration."""

    base: constr(min_length=1) = "python:3.10-bullseye"
    user: constr(min_length=1) = "ou"


class Server(BaseModel):
    """Settings for the core server configuration."""

    default_path: constr(min_length=1) = "/"
    access_token: str | None = None
    wrapper_host: str | None = None


class Content(BaseModel):
    """Settings for the content configuration."""

    source: constr(min_length=1)
    target: str = ""
    overwrite: Literal["always"] | Literal["never"] | Literal["if-unchanged"]


class AptDebLine(BaseModel):
    """Settings for an APT deb line."""

    url: HttpUrl
    distribution: constr(min_length=1)
    component: constr(min_length=1)


class AptSource(BaseModel):
    """Settings for a single APT source."""

    name: constr(min_length=1)
    key_url: HttpUrl
    dearmor: bool = True
    deb: AptDebLine


class Sources(BaseModel):
    """Settings for the additional sources configuration."""

    apt: list[AptSource] = []


class Packages(BaseModel):
    """Settings for the packages configuration."""

    apt: list[constr(min_length=1)] = []
    pip: list[constr(min_length=1)] = []


class Script(BaseModel):
    """Settings for a single script configuration."""

    commands: list[str] = []

    @validator("commands", pre=True)
    def convert_strings(cls: "Script", value: str | list[str]) -> list[str]:
        """Convert a string commands to a list."""
        if isinstance(value, str):
            return [line for line in value.split("\n") if line.strip()]
        else:
            return value


class Scripts(BaseModel):
    """Settings for the scripts configuration."""

    build: list[Script] = []
    startup: list[Script] = []
    shutdown: list[Script] = []


class LauncherEntry(BaseModel):
    """Settings for a single launcher entry."""

    enabled: bool = True
    icon_path: str = ""
    title: str = ""


class WebApp(BaseModel):
    """Settings for a single web application configuration."""

    path: constr(min_length=1)
    cmdline: list[constr(min_length=1)] | None = None
    command: list[constr(min_length=1)] | None = None
    port: int = 0
    default: bool = False
    timeout: int = 60
    absolute_url: bool = False
    environment: dict = {}
    new_browser_tab: bool = False
    request_headers_override: dict = {}
    launcher_entry: LauncherEntry = LauncherEntry()

    @validator("cmdline", pre=True)
    def convert_string_cmdline(cls: "WebApp", value: str | list[constr(min_length=1)]) -> list[constr(min_length=1)]:
        """Convert a string cmdline to its list representation."""
        if isinstance(value, str):
            return shlex.split(value)
        else:
            return value

    @validator("command", pre=True)
    def convert_string_command(cls: "WebApp", value: str | list[constr(min_length=1)]) -> list[constr(min_length=1)]:
        """Convert a string command to its list representation."""
        if isinstance(value, str):
            return shlex.split(value)
        else:
            return value


class EnvironmentVariable(BaseModel):
    """Settings for additional environment variables."""

    name: constr(min_length=1)
    value: str = ""


class Flags(BaseModel):
    """Settings for output flags."""

    ou_container_content: bool = False


class Settings(BaseModel):
    """Application Settings."""

    module: Module
    image: Image = Image()
    server: Server = Server()
    content: list[Content] = []
    sources: Sources = Sources()
    packages: Packages = Packages()
    scripts: Scripts = Script()
    web_apps: list[WebApp] = []
    services: list[constr(min_length=1)] = []
    environment: list[EnvironmentVariable] = []
    packs: list[JupyterLabPack | CodeServerPack | MariaDBPack | NBClassicPack | TutorialServerPack] = []
    hacks: list[Literal["missing-man1"]] = []
    flags: Flags = Flags()
    jupyter_server_config: dict = {}



"""Pack to setup and run system services.

Automatically generates a sudoers file that ensures that the default user can start and stop all services.
"""
import os

from jinja2 import Environment

from ou_container_builder.utils import merge_settings, render_template


def process_settings(settings: dict) -> dict:
    """Process the services core settings.

    :param settings: The settings parsed from the configuration file
    :type settings: dict
    :return: The updated settings
    :rtype: dict
    """
    settings = merge_settings(
        settings,
        {
            "content": [
                {
                    "source": "ou-builder-build/services/services.sudoers",
                    "target": "/etc/sudoers.d/99-services",
                    "overwrite": "always",
                }
            ],
            "flags": {"ou_container_content": True},
        },
    )
    return settings


def generate_files(context: str, env: Environment, settings: dict) -> None:
    """Generate the build files for the services core.

    :param context: The context path within which the generation is running
    :type context: str
    :param env: The Jinja2 environment to use for loading and rendering templates
    :type env: :class:`~jinja2.environment.Environment`
    :param settings: The validated settings
    :type settings: dict
    """
    render_template(
        context,
        env,
        "core/services/sudoers",
        os.path.join("ou-builder-build", "services", "services.sudoers"),
        settings,
    )



"""Test the docker args settings validation."""
from ou_container_builder.settings import DockerArg


def test_valid_docker_args_settings():
    """Test that a valid docker args configuration passes."""
    DockerArg(name="TARGETPLATFORM")
    DockerArg(name="TARGETPLATFORM", value="linux/amd64")



"""Pack to install the tutorial_sever."""
import os
from typing import Literal
from warnings import warn

from jinja2 import Environment
from pydantic import BaseModel

from ou_container_builder.utils import merge_settings, render_template


class Pack(BaseModel):
    name: Literal["tutorial_server"]
    options: dict = {}


def process_settings(settings: dict, pack_settings: dict) -> dict:
    """
    Process the user-provided settings.

    :param settings: The settings parsed from the configuration file
    :type settings: dict
    :param pack_settings: The pack-specific settings parsed from the configuration file
    :type settings: dict
    :return: The updated settings
    :rtype: dict
    """
    warn(
        "Automatically setting tutorial_server as the default is deprecated in 2.2.0 and will be removed in 3.0.0. Instead set server.default_path explicitly.",
        FutureWarning,
    )
    additional_settings = {
        "packages": {
            "pip": [
                "tutorial-server>=1.0.2,<2.0.0",
            ]
        },
        "content": [
            {
                "source": "ou-builder-build/tutorial-server/production.ini",
                "target": "/etc/tutorial-server/production.ini",
                "overwrite": "always",
            }
        ],
        "web_apps": [
            {
                "path": "/tutorial-server",
                "command": [
                    "python",
                    "-m",
                    "tutorial_server",
                    "--config=/etc/tutorial-server/production.ini",
                    "--port={port}",
                    "--basepath={base_url}tutorial-server/",
                ],
                "absolute_url": True,
                "default": True,
            }
        ],
    }
    if "php-cgi" in pack_settings and pack_settings["php-cgi"]:
        additional_settings["packages"]["apt"] = ["php-cgi"]
    settings = merge_settings(settings, additional_settings)
    return settings


def generate_files(context: str, env: Environment, settings: dict, pack_settings: dict) -> dict:
    """Generate the build files for the tutorial-server pack.

    Ensures that the Tutorial Server is installed and set up via the ``web_apps`` setting.

    :param context: The context path within which the generation is running
    :type context: str
    :param env: The Jinja2 environment to use for loading and rendering templates
    :type env: :class:`~jinja2.environment.Environment`
    :param settings: The validated settings
    :type settings: dict
    :param pack_settings: The validated pack-specific settings
    :type settings: dict
    """
    render_template(
        context,
        env,
        "packs/tutorial-server/production.ini",
        os.path.join("ou-builder-build", "tutorial-server", "production.ini"),
        dict(**settings, **pack_settings),
    )



"""Pack to handle setting up jupyter_server."""
import json
import os.path

from jinja2 import Environment

from ou_container_builder.utils import merge_settings


def process_settings(settings: dict) -> dict:
    """Process the jupyter_server core settings.

    :param settings: The settings parsed from the configuration file
    :type settings: dict
    :return: The updated settings
    :rtype: dict
    """
    settings = merge_settings(
        settings,
        {
            "packages": {"apt": ["sudo"], "pip": ["jupyter_server<4.0.0", "jupyterhub>=3.0.0,<4"]},
            "content": [
                {
                    "source": "ou-builder-build/ou_core_config.json",
                    "target": "/usr/local/etc/jupyter/jupyter_server_config.json",
                    "overwrite": "always",
                }
            ],
            "jupyter_server_config": {
                "ServerApp": {
                    "ip": "0.0.0.0",
                    "port": 8888,
                    "quit_button": False,
                    "trust_xheaders": True,
                    "iopub_data_rate_limit": 10000000,
                },
                "NotebookApp": {"quit_button": False},
            },
        },
    )
    if "access_token" in settings["server"] and settings["server"]["access_token"]:
        settings = merge_settings(
            settings, {"jupyter_server_config": {"ServerApp": {"token": settings["server"]["access_token"]}}}
        )
    if "wrapper_host" in settings["server"] and settings["server"]["wrapper_host"]:
        settings = merge_settings(
            settings,
            {
                "jupyter_server_config": {
                    "ServerApp": {
                        "tornado_settings": {
                            "headers": {
                                "Content-Security-Policy": f"frame-ancestors 'self' {settings['server']['wrapper_host']}",
                                "Access-Control-Allow-Origin": f'{settings["server"]["wrapper_host"]}',
                            }
                        }
                    }
                }
            },
        )
    return settings


def generate_files(context: str, env: Environment, settings: dict, user_settings) -> dict:
    """Generate the build files the jupyter_server core.

    :param context: The context path within which the generation is running
    :type context: str
    :param env: The Jinja2 environment to use for loading and rendering templates
    :type env: :class:`~jinja2.environment.Environment`
    :param settings: The validated settings
    :type settings: dict
    """
    if "server" in user_settings and "default_path" in user_settings["server"]:
        settings = merge_settings(
            settings, {"jupyter_server_config": {"ServerApp": {"default_url": user_settings["server"]["default_path"]}}}
        )
    with open(os.path.join(context, "ou-builder-build", "ou_core_config.json"), "w") as out_f:
        json.dump(settings["jupyter_server_config"], out_f)



"""Pack to handle web_apps setups.

This simply sets the server's default_path, if a web application is set as the default.
"""
from copy import deepcopy
from warnings import warn

from jinja2 import Environment

from ou_container_builder.utils import merge_settings


def process_settings(settings: dict) -> dict:
    """Process the env core settings.

    :param settings: The settings parsed from the configuration file
    :type settings: dict
    :return: The updated settings
    :rtype: dict
    """
    if len(settings["web_apps"]) > 0:
        settings = merge_settings(settings, {"packages": {"pip": ["jupyter-server-proxy>=3.2.1,<4.0.0"]}})
    servers = {}
    for web_app in settings["web_apps"]:
        if "cmdline" in web_app:
            if "command" in web_app:
                warn(
                    "Both cmdline and command specified for a web_app. As cmdline is deprecated in 2.2.0, command takes preceence.",
                    FutureWarning,
                )
            else:
                warn(
                    "The cmdline setting for a web_app is deprecated in 2.2.0 and will be removed for version 3.0.0. Use command instead.",
                    FutureWarning,
                )
                web_app["command"] = web_app["cmdline"]
            del web_app["cmdline"]
        elif "command" not in web_app:
            warn("No command specified for the web_app")
        if "default" in web_app:
            warn(
                "The default setting in the web_app configuration is deprecated in 2.2.0 and will be removed in 3.0.0. Use server.default_path instead",
                FutureWarning,
            )
            if web_app["default"]:
                settings = merge_settings(settings, {"server": {"default_path": web_app["path"]}})
            del web_app["default"]
        path = web_app["path"]
        servers[path] = deepcopy(web_app)
        del servers[path]["path"]
    settings = merge_settings(settings, {"jupyter_server_config": {"ServerProxy": {"servers": servers}}})
    return settings


def generate_files(context: str, env: Environment, settings: dict) -> None:
    """Generate the build files for the web_apps core.

    :param context: The context path within which the generation is running
    :type context: str
    :param env: The Jinja2 environment to use for loading and rendering templates
    :type env: :class:`~jinja2.environment.Environment`
    :param settings: The validated settings
    :type settings: dict
    """
    pass



"""The main OU Container Builder commandline application.

Run ``ou-container-builder --help`` for help with the command-line parameters.
"""
import os

import click
from pydantic import ValidationError
from yaml import safe_load

from ou_container_builder.builder import run_build


@click.command()
@click.option(
    "-c", "--context", default=".", help="Context within which the container will be built", show_default=True
)
@click.option("-b/-nb", "--build/--no-build", default=True, help="Automatically build the container", show_default=True)
@click.option(
    "--clean/--no-clean", default=True, help="Automatically clean up after building the container", show_default=True
)
@click.option("--tag", multiple=True, help="Automatically tag the generated image")
def main(context: str, build: bool, clean: bool, tag: list):
    """Build your OU Container."""
    with open(os.path.join(context, "ContainerConfig.yaml")) as config_f:
        settings = safe_load(config_f)
    try:
        run_build(settings, context, build, clean, tag)
    except ValidationError as ve:
        click.echo(click.style("There are errors in your configuration settings:", fg="red"), err=True)
        click.echo(err=True)
        for error in ve.errors():
            click.echo(click.style(f"{'.'.join([str(v) for v in error['loc']])}: {error['msg']}", fg="red"), err=True)


if __name__ == "__main__":
    main()
